from django.apps import AppConfig


class KatkaCoreConfig(AppConfig):
    name = "katka"
    label = "katka"

    def ready(self):
        super().ready()
        # import signals so the signal handlers are registered
        from . import signals  # noqa: F401



# Generated by Django 2.2.6 on 2019-10-08 08:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0019_scmsteprun_sequence_id"),
    ]

    operations = [
        migrations.RemoveField(model_name="scmrelease", name="scm_pipeline_run",),
        migrations.AddField(
            model_name="scmrelease", name="scm_pipeline_runs", field=models.ManyToManyField(to="katka.SCMPipelineRun"),
        ),
        migrations.AddField(
            model_name="scmrelease",
            name="status",
            field=models.CharField(choices=[("open", "open"), ("closed", "closed")], default="open", max_length=30),
        ),
    ]



import logging

from django.conf import settings
from django.db.models.signals import post_save
from django.dispatch import receiver

from katka.constants import (
    PIPELINE_STATUS_IN_PROGRESS,
    PIPELINE_STATUS_INITIALIZING,
    PIPELINE_STATUS_QUEUED,
    PIPELINE_STATUS_SKIPPED,
    STEP_FINAL_STATUSES,
)
from katka.fields import username_on_model
from katka.models import SCMPipelineRun, SCMStepRun
from katka.releases import close_release_if_pipeline_finished, create_release_if_necessary
from requests import HTTPError

log = logging.getLogger("katka")


@receiver(post_save, sender=SCMStepRun)
def update_pipeline_from_steps(sender, **kwargs):
    """
    Update the pipeline 'steps_completed' and 'steps_total' in case they changed whenever a step is updated/added
    """
    pipeline = kwargs["instance"].scm_pipeline_run
    pipeline_steps = SCMStepRun.objects.filter(scm_pipeline_run=pipeline)

    before_steps_total = pipeline.steps_total
    before_steps_completed = pipeline.steps_completed

    pipeline.steps_total = pipeline_steps.count()
    pipeline.steps_completed = pipeline_steps.filter(status__in=STEP_FINAL_STATUSES).count()

    if pipeline.steps_completed != before_steps_completed or pipeline.steps_total != before_steps_total:
        with username_on_model(SCMPipelineRun, kwargs["instance"].modified_username):
            pipeline.save()


@receiver(post_save, sender=SCMPipelineRun)
def send_pipeline_change_notification(sender, **kwargs):
    pipeline = kwargs["instance"]
    if pipeline.status == PIPELINE_STATUS_INITIALIZING and kwargs["created"] is False:
        # Do not send notifications when the pipeline is initializing. While initializing, steps are created and
        # since this is done with several requests, several notifications would be sent, while the only one you
        # care about is when all the steps are created and the status is changed to 'in progress'.
        # There is one exception though, a notify *should* be sent when the pipeline is first created, because
        # the notification will trigger the creation of the steps.
        return

    if pipeline.status == PIPELINE_STATUS_QUEUED:
        # When a pipeline is queued it means that other pipelines should be run first. To prevent them from
        # being run, do not notify.
        return

    session = settings.PIPELINE_CHANGE_NOTIFICATION_SESSION
    response = session.post(
        settings.PIPELINE_CHANGE_NOTIFICATION_URL, json={"public_identifier": str(pipeline.public_identifier)}
    )

    try:
        response.raise_for_status()
    except HTTPError:
        log.exception("Failed to notify pipeline runner")


@receiver(post_save, sender=SCMPipelineRun)
def create_close_releases(sender, **kwargs):
    pipeline = kwargs["instance"]
    if pipeline.status == PIPELINE_STATUS_SKIPPED:
        return

    if pipeline.status == PIPELINE_STATUS_IN_PROGRESS:
        create_release_if_necessary(pipeline)
    else:
        close_release_if_pipeline_finished(pipeline)



# Generated by Django 2.1.7 on 2019-04-25 09:22

import django.db.models.deletion
from django.db import migrations, models

import katka.fields


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0015_update_pipeline_statuses_fix"),
    ]

    operations = [
        migrations.CreateModel(
            name="ApplicationMetadata",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("created_username", katka.fields.AutoUsernameField(max_length=50)),
                ("modified", models.DateTimeField(auto_now=True)),
                ("modified_username", katka.fields.AutoUsernameField(max_length=50)),
                ("deleted", models.BooleanField(default=False)),
                ("key", models.CharField(max_length=50)),
                ("value", models.CharField(max_length=200)),
                ("application", models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="katka.Application")),
            ],
        ),
        migrations.AlterUniqueTogether(name="applicationmetadata", unique_together={("application", "key")},),
    ]



# Generated by Django 2.2.7 on 2019-12-30 09:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0029_release_pipeline_run_created_at_ordering"),
    ]

    operations = [
        migrations.AlterUniqueTogether(name="credential", unique_together={("name", "team")},),
    ]



# Generated by Django 2.2.6 on 2019-11-04 02:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0023_auto_20191024_0825"),
    ]

    operations = [
        migrations.AddField(model_name="scmsteprun", name="ended", field=models.DateTimeField(blank=True, null=True),),
        migrations.AddField(
            model_name="scmsteprun", name="started", field=models.DateTimeField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.6 on 2019-11-15 10:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0027_commit_ordering"),
    ]

    operations = [
        migrations.AddConstraint(
            model_name="scmpipelinerun",
            constraint=models.UniqueConstraint(
                fields=("commit_hash", "application"), name="unique commits per application"
            ),
        ),
    ]



import logging
from datetime import datetime

from django.conf import settings
from django.db import IntegrityError

import pytz
from katka.constants import (
    PIPELINE_FINAL_STATUSES,
    PIPELINE_STATUS_IN_PROGRESS,
    PIPELINE_STATUS_INITIALIZING,
    PIPELINE_STATUS_QUEUED,
    PIPELINE_STATUS_SKIPPED,
    STEP_FINAL_STATUSES,
)
from katka.exceptions import AlreadyExists, ParentCommitMissing
from katka.models import (
    Application,
    ApplicationMetadata,
    Credential,
    CredentialSecret,
    Project,
    SCMPipelineRun,
    SCMRelease,
    SCMRepository,
    SCMService,
    SCMStepRun,
    Team,
)
from katka.serializers import (
    ApplicationMetadataSerializer,
    ApplicationSerializer,
    CredentialSecretSerializer,
    CredentialSerializer,
    ProjectSerializer,
    SCMPipelineRunSerializer,
    SCMReleaseSerializer,
    SCMRepositorySerializer,
    SCMServiceSerializer,
    SCMStepRunSerializer,
    SCMStepRunUpdateSerializer,
    TeamSerializer,
)
from katka.viewsets import AuditViewSet, FilterViewMixin, ReadOnlyAuditMixin, UpdateAuditMixin

log = logging.getLogger(__name__)


class TeamViewSet(FilterViewMixin, AuditViewSet):
    model = Team
    serializer_class = TeamSerializer
    lookup_field = "public_identifier"
    lookup_value_regex = "[0-9a-f-]{36}"
    parameter_lookup_map = {"application": "project__application"}

    def get_user_restricted_queryset(self, queryset):
        # Only show teams that are linked to a group that the user is part of
        user_groups = self.request.user.groups.all()
        return queryset.filter(group__in=user_groups)


class ProjectViewSet(FilterViewMixin, AuditViewSet):
    model = Project
    serializer_class = ProjectSerializer

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        return queryset.filter(team__group__in=user_groups)


class ApplicationViewSet(FilterViewMixin, AuditViewSet):
    model = Application
    serializer_class = ApplicationSerializer

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        return queryset.filter(project__team__group__in=user_groups)


class CredentialViewSet(FilterViewMixin, AuditViewSet):
    model = Credential
    serializer_class = CredentialSerializer

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        return queryset.filter(team__group__in=user_groups)


class CredentialSecretsViewSet(AuditViewSet):
    model = CredentialSecret
    serializer_class = CredentialSecretSerializer
    lookup_field = "key"

    def get_queryset(self):
        kwargs = {
            "credential__deleted": False,
            "credential": self.kwargs["credentials_pk"],
        }

        return super().get_queryset().filter(**kwargs)

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        return queryset.filter(credential__team__group__in=user_groups)


class SCMServiceViewSet(ReadOnlyAuditMixin):
    model = SCMService
    serializer_class = SCMServiceSerializer

    def get_user_restricted_queryset(self, queryset):
        return queryset  # no restrictions on user, this is global information


class SCMRepositoryViewSet(FilterViewMixin, AuditViewSet):
    model = SCMRepository
    serializer_class = SCMRepositorySerializer

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        return queryset.filter(credential__team__group__in=user_groups)


class SCMPipelineRunViewSet(FilterViewMixin, AuditViewSet):
    model = SCMPipelineRun
    serializer_class = SCMPipelineRunSerializer

    parameter_lookup_map = {
        "scmrelease": "scmrelease",
        "release": "scmrelease",
    }

    def _can_create(self, application, parent_hash):
        if parent_hash is None:
            return True  # this is probably the first commit

        try:
            self.model.objects.get(application=application, commit_hash=parent_hash)
        except self.model.DoesNotExist:
            return False  # parent commit does not exist, we need a sync before we can continue

        return True

    def create(self, request, *args, **kwargs):
        try:
            return super().create(request, *args, **kwargs)
        except IntegrityError:
            raise AlreadyExists()

    def perform_create(self, serializer):
        application = serializer.validated_data["application"]
        if not self._can_create(application, serializer.validated_data.get("first_parent_hash")):
            raise ParentCommitMissing()

        # Upon pipeline creation, set to skipped if the application is inactive and no status is provided
        status = serializer.validated_data.get("status", None)
        if status is None and not application.active:
            serializer.validated_data["status"] = PIPELINE_STATUS_SKIPPED

        super().perform_create(serializer)

    def _ready_to_run(self, pipeline_run):
        if pipeline_run.first_parent_hash is None:
            return True  # this is probably the first commit

        try:
            parent_instance = self.model.objects.get(
                application=pipeline_run.application, commit_hash=pipeline_run.first_parent_hash
            )
        except self.model.DoesNotExist:
            # parent commit does not exist, can't run. This should not happen since we are only creating pipeline runs
            # with a defined parent commit
            log.exception(
                f"Parent pipeline run with hash {pipeline_run.first_parent_hash} could not "
                f"be found for pipeline run {pipeline_run.public_identifier}. This should not be the case since "
                f"pipeline runs with no parent are not added to the DB (exception being the initial commit)."
            )
            return False

        if parent_instance.status not in PIPELINE_FINAL_STATUSES:
            return False  # exists, but not ready to run this one yet

        return True

    def perform_update(self, serializer):
        status = serializer.validated_data.get("status", None)
        if status == PIPELINE_STATUS_IN_PROGRESS:
            if not self._ready_to_run(serializer.instance):
                serializer.validated_data["status"] = PIPELINE_STATUS_QUEUED

        super().perform_update(serializer)

        if status in PIPELINE_FINAL_STATUSES:
            self._run_next_pipeline_if_available(serializer.instance.application, serializer.instance.commit_hash)

    def _run_next_pipeline_if_available(self, application, commit_hash):
        try:
            next_pipeline = self.model.objects.get(application=application, first_parent_hash=commit_hash)
        except self.model.DoesNotExist:
            return  # does not exist (yet), so nothing to do

        if next_pipeline.status != PIPELINE_STATUS_QUEUED:
            if next_pipeline.status != PIPELINE_STATUS_INITIALIZING:
                log.warning(
                    f'Next pipeline {next_pipeline.pk} is not queued, it has status "{next_pipeline.status}", '
                    "not updating"
                )
            return

        # No need to wrap inside a "with username_on_model():" context manager since at this point we already
        # are in a context. Worried about infinite recursion because we keep setting statuses for the next
        # pipelines? Since this method is only called when a pipeline moves from the "in progress" state to
        # a final state and the next pipeline is set to the "in progress" state, it should not trigger recursion.
        next_pipeline.status = PIPELINE_STATUS_IN_PROGRESS
        next_pipeline.save()

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        return queryset.filter(application__project__team__group__in=user_groups)


class QueuedSCMPipelineRunViewSet(FilterViewMixin, ReadOnlyAuditMixin):
    model = SCMPipelineRun
    serializer_class = SCMPipelineRunSerializer

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        return queryset.filter(
            application__project__team__group__in=user_groups, status=PIPELINE_STATUS_QUEUED, scmrelease__isnull=True
        )


def pre_validate_steprun_update(serializer):
    """In case of an update to the status which sets the step to a final status, ensure the 'ended_at'
       field is set.
    """
    if (
        "status" in serializer.validated_data
        and serializer.validated_data["status"] in STEP_FINAL_STATUSES
        and "ended_at" not in serializer.validated_data
    ):
        serializer.validated_data["ended_at"] = datetime.now(tz=pytz.timezone(settings.TIME_ZONE))


class SCMStepRunViewSet(FilterViewMixin, AuditViewSet):
    model = SCMStepRun
    serializer_class = SCMStepRunSerializer

    def perform_update(self, serializer):
        pre_validate_steprun_update(serializer)
        serializer.save()

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        return queryset.filter(scm_pipeline_run__application__project__team__group__in=user_groups)


class SCMStepRunUpdateStatusView(UpdateAuditMixin):
    model = SCMStepRun
    serializer_class = SCMStepRunUpdateSerializer

    def perform_update(self, serializer):
        pre_validate_steprun_update(serializer)
        serializer.save()

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        return queryset.filter(scm_pipeline_run__application__project__team__group__in=user_groups)


class SCMReleaseViewSet(FilterViewMixin, ReadOnlyAuditMixin):
    model = SCMRelease
    serializer_class = SCMReleaseSerializer

    parameter_lookup_map = {"application": "scm_pipeline_runs__application", "pipeline_run": "scm_pipeline_runs"}

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        # Do select distinct because of the many to many relationship
        return queryset.distinct().filter(scm_pipeline_runs__application__project__team__group__in=user_groups)


class ApplicationMetadataViewSet(AuditViewSet):
    model = ApplicationMetadata
    serializer_class = ApplicationMetadataSerializer
    lookup_field = "key"

    def get_queryset(self):
        kwargs = {
            "application__deleted": False,
            "application": self.kwargs["applications_pk"],
            "deleted": False,
        }

        return super().get_queryset().filter(**kwargs)

    def get_user_restricted_queryset(self, queryset):
        user_groups = self.request.user.groups.all()
        return queryset.filter(application__project__team__group__in=user_groups)



# Generated by Django 2.2.7 on 2020-01-06 12:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0030_unique_team_credentialname"),
    ]

    operations = [
        migrations.AddField(model_name="application", name="active", field=models.BooleanField(default=True),),
    ]



# Generated by Django 2.1.7 on 2019-03-18 08:06

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0010_rename_scmtype"),
    ]

    operations = [
        migrations.AlterUniqueTogether(name="credential", unique_together=set(),),
        migrations.RemoveField(model_name="credential", name="slug",),
    ]



from django.utils.translation import gettext_lazy as _

from rest_framework import status
from rest_framework.exceptions import APIException


class MissingUsername(Exception):
    pass


class Conflict(APIException):
    status_code = status.HTTP_409_CONFLICT
    default_detail = _("Conflict.")
    default_code = "conflict"


class AlreadyExists(Conflict):
    default_detail = {"detail": "Object already exists.", "code": "already_exists"}
    default_code = "already_exists"


class ParentCommitMissing(Conflict):
    default_detail = {
        "detail": "Pipeline Run could not be created, no pipeline run found for parent commit hash.",
        "code": "parent_commit_missing",
    }
    default_code = "parent_commit_missing"



# Generated by Django 2.1.5 on 2019-04-14 11:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0012_scmrelease"),
    ]

    operations = [
        migrations.AlterField(
            model_name="scmsteprun",
            name="status",
            field=models.CharField(
                choices=[
                    ("not started", "not started"),
                    ("in progress", "in progress"),
                    ("waiting", "waiting"),
                    ("skipped", "skipped"),
                    ("failed", "failed"),
                    ("success", "success"),
                ],
                default="in progress",
                max_length=30,
            ),
        ),
    ]



from katka.auth import has_full_access_scope
from katka.models import Application, Credential, Project, SCMPipelineRun, SCMRepository, SCMService, Team
from rest_framework import serializers
from rest_framework.exceptions import NotFound, PermissionDenied
from rest_framework.relations import PrimaryKeyRelatedField


class GroupNameField(serializers.RelatedField):
    def to_internal_value(self, group_name):
        qs = self.get_queryset()
        try:
            return qs.get(name=group_name)
        except qs.model.DoesNotExist:
            raise NotFound

    def to_representation(self, group):
        return group.name


class PrimaryKeyRelated403Field(PrimaryKeyRelatedField):
    """
    Instead of replying with a 400 with a message 'does_not_exist', we would rather tell the user that they
    do not have permission because the referred team/credential/etc. either does not exist or they are not
    a member of a group that is linked to that object.
    """

    def fail(self, key, **kwargs):
        if key == "does_not_exist":
            raise PermissionDenied(self.does_not_exist_message)

        return super().fail(key, **kwargs)


class TeamRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = "Team does not exist or you are not a member of its group"

    def get_queryset(self):
        request = self.context["request"]
        queryset = Team.objects.filter(deleted=False)
        if not has_full_access_scope(request):
            queryset = queryset.filter(group__in=request.user.groups.all())

        return queryset


class ProjectRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = "Project does not exist or is not linked to your team"

    def get_queryset(self):
        request = self.context["request"]
        queryset = Project.objects.filter(team__deleted=False, deleted=False)
        if not has_full_access_scope(request):
            queryset = queryset.filter(team__group__in=request.user.groups.all())

        return queryset


class CredentialRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = "Credential or team does not exist or is not linked to your group"

    def get_queryset(self):
        request = self.context["request"]
        queryset = Credential.objects.filter(team__deleted=False, deleted=False)
        if not has_full_access_scope(request):
            queryset = queryset.filter(team__group__in=request.user.groups.all())

        return queryset


class SCMServiceRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = "SCMService does not exist"

    def get_queryset(self):
        return SCMService.objects.filter(deleted=False)


class SCMRepositoryRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = "SCMRepository does not exist"

    def get_queryset(self):
        return SCMRepository.objects.filter(scm_service__deleted=False, credential__deleted=False, deleted=False)


class ApplicationRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = "Application does not exist or does not belong to your team"

    def get_queryset(self):
        request = self.context["request"]
        queryset = Application.objects.filter(project__team__deleted=False, project__deleted=False, deleted=False,)
        if not has_full_access_scope(request):
            queryset = queryset.filter(project__team__group__in=request.user.groups.all())

        return queryset


class SCMPipelineRunRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = "SCM Pipeline Run does not exist or does not belong to your team"

    def get_queryset(self):
        request = self.context["request"]
        queryset = SCMPipelineRun.objects.filter(
            application__project__team__deleted=False,
            application__project__deleted=False,
            application__deleted=False,
            deleted=False,
        )
        if not has_full_access_scope(request):
            queryset = queryset.filter(application__project__team__group__in=request.user.groups.all())

        return queryset



from django.contrib import admin

from katka.fields import username_on_model
from katka.models import (
    Application,
    ApplicationMetadata,
    Credential,
    CredentialSecret,
    Project,
    SCMPipelineRun,
    SCMRelease,
    SCMRepository,
    SCMService,
    SCMStepRun,
    Team,
)


class ReadOnlyAuditFieldsMixin:
    readonly_fields = ("created_at", "created_username", "modified_at", "modified_username", "deleted")


class WithUsernameAdminModel(admin.ModelAdmin, ReadOnlyAuditFieldsMixin):
    def save_model(self, request, obj, form, change):
        with username_on_model(self.model, request.user.username):
            super().save_model(request, obj, form, change)


@admin.register(Team)
class TeamAdmin(WithUsernameAdminModel):
    fields = ("name", "slug", "group")


@admin.register(Project)
class ProjectAdmin(WithUsernameAdminModel):
    fields = ("name", "slug", "team")


@admin.register(Application)
class ApplicationAdmin(WithUsernameAdminModel):
    fields = ("name", "slug", "project", "scm_repository", "active")


@admin.register(Credential)
class CredentialAdmin(WithUsernameAdminModel):
    fields = ("name", "credential_type", "team")


@admin.register(CredentialSecret)
class CredentialSecretAdmin(WithUsernameAdminModel):
    fields = ("key", "value", "credential")


@admin.register(SCMService)
class SCMServiceAdmin(WithUsernameAdminModel):
    fields = ("scm_service_type", "server_url")


@admin.register(SCMRepository)
class SCMRepositoryAdmin(WithUsernameAdminModel):
    fields = ("organisation", "repository_name", "credential", "scm_service")
    list_display = ("pk", "scm_service", "organisation", "repository_name")


@admin.register(SCMPipelineRun)
class SCMPipelineRunAdmin(WithUsernameAdminModel):
    fields = (
        "commit_hash",
        "first_parent_hash",
        "status",
        "steps_total",
        "steps_completed",
        "pipeline_yaml",
        "application",
    )
    list_display = ("pk", "application", "commit_hash", "first_parent_hash", "created_at")
    list_filter = ("application__name",)
    ordering = ("created_at",)


@admin.register(SCMStepRun)
class SCMStepRunAdmin(WithUsernameAdminModel):
    fields = ("slug", "name", "stage", "status", "output", "started_at", "ended_at", "scm_pipeline_run")
    list_display = ("pk", "scm_pipeline_run", "name", "stage", "status", "created_at", "started_at", "ended_at")
    list_filter = ("scm_pipeline_run__application__name",)
    ordering = ("created_at",)


@admin.register(ApplicationMetadata)
class ApplicationMetadataAdmin(WithUsernameAdminModel):
    fields = ("key", "value", "application")
    list_display = ("pk", "application", "key")


@admin.register(SCMRelease)
class SCMReleaseAdmin(WithUsernameAdminModel):
    fields = ("name", "status", "started_at", "ended_at", "scm_pipeline_runs")
    list_display = ("pk", "name", "status", "created_at", "started_at", "ended_at")
    ordering = ("created_at",)



# Generated by Django 2.2.6 on 2019-10-21 06:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0020_multiple_pipeline_runs_per_release"),
    ]

    operations = [
        migrations.AddField(model_name="scmsteprun", name="tags", field=models.TextField(blank=True),),
    ]



# Generated by Django 2.1.5 on 2019-05-06 04:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0017_auto_20190501_0917"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="scmpipelinerun", options={"verbose_name": "SCM pipeline", "verbose_name_plural": "SCM pipelines"},
        ),
        migrations.AlterModelOptions(
            name="scmrelease", options={"verbose_name": "SCM release", "verbose_name_plural": "SCM releases"},
        ),
        migrations.AlterModelOptions(
            name="scmrepository", options={"verbose_name": "SCM repository", "verbose_name_plural": "SCM repositories"},
        ),
        migrations.AlterModelOptions(
            name="scmservice", options={"verbose_name": "SCM service", "verbose_name_plural": "SCM services"},
        ),
        migrations.AlterModelOptions(
            name="scmsteprun", options={"verbose_name": "SCM step", "verbose_name_plural": "SCM steps"},
        ),
    ]



# Generated by Django 2.1.5 on 2019-05-01 09:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0016_auto_20190425_0922"),
    ]

    operations = [
        migrations.AlterField(
            model_name="scmpipelinerun", name="pipeline_yaml", field=models.TextField(default="---"),
        ),
        migrations.AlterField(
            model_name="scmpipelinerun", name="steps_total", field=models.PositiveSmallIntegerField(default=0),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-07 14:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0025_datetime_field_names"),
    ]

    operations = [
        migrations.AddField(
            model_name="scmsteprun", name="step_type", field=models.CharField(max_length=100, null=True),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-09 02:09

import uuid

import django.db.models.deletion
from django.db import migrations, models

import katka.fields


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0011_auto_20190318_0806"),
    ]

    operations = [
        migrations.CreateModel(
            name="SCMRelease",
            fields=[
                ("created", models.DateTimeField(auto_now_add=True)),
                ("created_username", katka.fields.AutoUsernameField(max_length=50)),
                ("modified", models.DateTimeField(auto_now=True)),
                ("modified_username", katka.fields.AutoUsernameField(max_length=50)),
                ("deleted", models.BooleanField(default=False)),
                (
                    "public_identifier",
                    models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                ("name", models.CharField(max_length=255)),
                ("released", models.DateTimeField(null=True)),
                ("from_hash", models.CharField(max_length=64)),
                ("to_hash", models.CharField(max_length=64)),
                (
                    "scm_pipeline_run",
                    models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to="katka.SCMPipelineRun"),
                ),
            ],
            options={"abstract": False,},
        ),
    ]



# Generated by Django 2.1.5 on 2019-02-07 08:12

import uuid

import django.db.models.deletion
from django.db import migrations, models

import katka.fields


class Migration(migrations.Migration):

    dependencies = [
        ("katka", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Project",
            fields=[
                ("created", models.DateTimeField(auto_now_add=True)),
                ("created_username", katka.fields.AutoUsernameField(max_length=50)),
                ("modified", models.DateTimeField(auto_now=True)),
                ("modified_username", katka.fields.AutoUsernameField(max_length=50)),
                (
                    "status",
                    models.CharField(
                        choices=[("active", "active"), ("inactive", "inactive")], default="active", max_length=50
                    ),
                ),
                (
                    "public_identifier",
                    models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                ("slug", katka.fields.KatkaSlugField(max_length=10, unique=True)),
                ("name", models.CharField(max_length=100)),
            ],
            options={"abstract": False,},
        ),
        migrations.AddField(
            model_name="team",
            name="slug",
            field=katka.fields.KatkaSlugField(default="NONE", max_length=10, unique=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="project",
            name="team",
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="katka.Team"),
        ),
    ]



from enum import Enum, auto

from django.conf import settings


class AuthType(Enum):
    ANONYMOUS = auto()
    GROUPS = auto()
    SCOPES = auto()


def has_full_access_scope(request):
    scopes = getattr(request, "scopes", ())
    if getattr(settings, "SCOPE_FULL_ACCESS", None) in scopes:
        return True

    return False
